;Sprite Drawing Routine (SDR): 8 bit version, clipping
;by JL
;<JayEll64@aol.com>
;
;Inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;Outputs (if sprite or partial sprite drawn):
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=pointer to byte after sprite
;	no other registers are changed
;	(If you want to save these registers, push them *before* the call
;	 and pop them back *after* the call, not *in* the call itself!)
;
;Sprite data should be stored as follows:
;.db (# rows/bytes/height of sprite)
;.db (bitmap of sprite)
;
;Size=110 bytes (+pushes/pops)
;
;Notes:
;You can 'xor' the sprite to the video memory by changing both 'or (hl)'s
; with 'xor (hl)'s
;Also, if all your sprites are the same height, you can delete the next 2
; lines (ld e,(hl)\inc hl); change the 3rd line after that (ld c,e) to
; 'ld c,x', where x is the height of your sprites; and store all sprites
; with only the sprite data, leaving off that first byte that defines the
; height (saves 1 bytes + # of sprites!)
;It shouldn't be too hard to make white transparent in this routine; just
; refer to SDR8WT.H and SDR8.H to see what needs to be done.
;If you want the sprite to be drawn in the graphics memory instead of the
; video memory, simply replace the one instance of VIDEO_MEM with
; GRAPH_MEM.
;
;***THIS ROUTINE ASSUMES THAT THE ORIGIN (0,0) IS THE *BOTTOM* LEFT CORNER!
; YOU'LL HAVE TO CHANGE THE ROUTINE SLIGHTLY IF YOU WANT THE ORIGIN TO BE
; THE TOP LEFT CORNER (JUST THE CLIPPING PART, I THINK)!***
;
;***You also need to define a byte labeled "ClipFlag".***

SDR8CL:
 ld e,(hl)	;-> can be replaced with 'ld e,8' if all your sprites are
 inc hl		;/	8x8
 xor a	;a=0
 ld (ClipFlag),a	;flag=0 (meaning non-clipping)

 ld a,c	;a=y coor
 inc a	;a=y coor+1
 cp e	;compare to number of rows to display
 jr nc,SDR8_CheckTClip	;if no carry, no bottom clipping
 ld e,a	;else, load y coor+1 into row counter
SDR8_CheckTClip:
 sub 65	;a=65-a	(checks for top clipping)
 jr c,SDR8_CheckLClip	;if a carry, no top clipping
 inc a	;else, a=a+1
 ld d,a	;save into counter
 sub e	;subtract number of rows to put
 ret nc	;if no carry, sprite won't even show up
 neg	;make a positive
 ld e,a	;and that's the number of rows to display
 ld c,63	;and here's the y coor we're going to start from
SDR8_PrepHL:
 inc hl	;next byte down in sprite
 dec d	;decrease counter
 jr nz,SDR8_PrepHL	;and repeat

SDR8_CheckLClip:	;now check left clipping
 ld a,b	;a=x coor
 cp 128	;check if start of sprite drawing is within screen (0-127)
 jr c,SDR8_DoneClip ;if so, done clipping (right clipping is "automatic")
 cp -7	;check if starts too far left
 ret c	;if so, sprite won't show
 inc c	;increase y coor (move it up one)
 add a,128	;add 128 to x coor to make it start on right side and wrap
 ld b,a	;save into x coor
 ld (ClipFlag),a	;and make flag non-zero

SDR8_DoneClip:
 push hl	;push pointer to sprite to stack
 call FIND_PIXEL	;)
 ld c,e	;c=number of rows in sprite
 ld de,VIDEO_MEM	;define offset
 add hl,de	;and add offset to get where to put sprite

 add a,a	;-> sets all bits to the right of the set bit (sprite mask)
 dec a		;/
 ld e,a	;save sprite mask into e
 ld a,b	;get x coor
 and %00000111	;find remainder of x/8
 inc a	;increase to get range of 1-8
 ld b,a	;b=rotate counter

SDR8CL_NewRow:
 ex (sp),hl	;hl(video location)<->(sp)(pointer to sprite)
 ld a,(hl)	;a=byte in sprite
 inc hl	;next byte
 ex (sp),hl	;hl<->(sp)
 push bc	;save rotate counter and row counter

 rlca	;rotate left once...
SDR8CL_PrepByte:
 rrca	;...then rotate right...
 djnz SDR8CL_PrepByte	;...b times

 ld b,a	;save rotated sprite byte into b
 ld a,(ClipFlag)	;a=left clipping flag
 or a	;check if not 0
 jr nz,SDR8CL_SkipLeft	;if left clipping, only display right half of sprite
 ld a,b	;else, load rotated sprite byte back into a
 and e	;mask off left bits
 ld d,a	;(check SDR8 to see how this works...)
 ld a,e
 cpl
 and (hl)
 or d
 ld (hl),a
 inc hl

 ld a,l	;a=lower byte of video location to put sprite
 and %00001111	;mask off upper nibble
 jr z,SDR8CL_NextRow ;if zero, hl now points to a new row, so right clipping
 dec hl	;else, decrease hl to put it back
SDR8CL_SkipLeft:
 inc hl	;then increase it for possible left clipping

 ld a,e	;(again, check SDR8 for this explanation)
 cpl
 and b
 ld d,a
 ld a,e
 and (hl)
 or d
 ld (hl),a

SDR8CL_NextRow:
 ld bc,$10-1	;bc=number of bytes in one row-1
 add hl,bc	;add to point hl to next row
 pop bc	;restore rotate counter and row counter
 dec c	;decrease row counter
 jr nz,SDR8CL_NewRow	;and repeat
 pop hl	;restore pointer to sprite (now points to byte after sprite)
 ret	;return