;Sprite Drawing Routine (SDR): 8 Bit White Transparency Version
;by JL
;<JayEll64@aol.com>
;
;Inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;Outputs:
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=pointer to byte after sprite
;	no other registers are changed
;If you want to save those registers that are destroyed, push/pop them!
;
;Sprite data should be stored as follows:
;.db (# rows/bytes/height of sprite)
;.db (bitmap of sprite)
;
;Size=52 bytes (+pushes/pops)
;Maybe the smallest sprite routine around???
;
;Notes:
;You can 'xor' the sprite to the video memory by changing both 'or (hl)'s
; with 'xor (hl)'s
;You can make this routine smaller by deleting the first 3 lines and
; accounting in your program the fact the the origin (0,0) is the *bottom*
; left corner (saves 4 bytes!).
;Also, if all your sprites are the same height, you can delete the next 2
; lines (ld e,(hl)\inc hl); change the 3rd line after that (ld c,e) to
; 'ld c,x', where x is the height of your sprites; and store all sprites
; with only the sprite data, leaving off that first byte that defines the
; height (saves 1 bytes + # of sprites!)

SDR8WT:
 ld a,63	;--> move origin to bottom left if in program origin is top
 sub c		; /	left
 ld c,a		;/   may be deleted if accounted for in program
 ld e,(hl)	;e=number of rows in sprite ;may be deleted as above
 inc hl	;hl now points to actual sprite ;may be deleted as above
 push hl	;save hl to the stack
 call FIND_PIXEL	;make a ROM call
 ld c,e	;c=number of rows in sprite ;may be changed as above
 ld de,VIDEO_MEM	;offset to display sprite
 add hl,de	;and add that offset

 add a,a	;-> set all bits to the right of the set bit in a
 dec a		;/	for example if a=%00001000 before, a=%00001111 aft
 ld e,a	;save mask into e
 ld a,b	;a=x coor
 and %00000111	;find remainder of x/8 (last 3 bits hold the remainder)
 inc a	;now a is in the range of 1 through 8
 ld b,a	;and save rotate counter into b

SDR8WT_NewRow:
 ex (sp),hl	;exchange word in stack (pointer to sprite) with hl
 ld a,(hl)	;load in sprite byte
 inc hl	;next byte in sprite
 ex (sp),hl	;exchange word in stack with hl (video location)
 push bc	;save rotate counter and row counter

 rlca	;rotate a one to the left
SDR8WT_PrepByte:
 rrca	;now rotate a b times to the right
 djnz SDR8WT_PrepByte	;(repeat b times)

 ld b,a	;save rotated sprite byte into b
 and e	;mask off left bits
 or (hl)	;'or' with current byte at video location
 ld (hl),a	;and display half of the sprite
 inc hl	;next byte over

 ld a,e	;a=mask
 cpl	;complement to...
 and b	;mask off right bits in rotated sprite byte
 or (hl)	;'or' it with what's already there
 ld (hl),a	;and put other half of sprite row

 ld bc,$10-1	;bc=number of bytes 1 row is minus 1
 add hl,bc	;add to point hl to next row
 pop bc	;restore rotate counter and row counter
 dec c	;decrease row counter
 jr nz,SDR8WT_NewRow	;and repeat until sprite is displayed
 pop hl	;restore pointer to sprite (now points to after sprite)
 ret	;return...